import 'package:flutter/material.dart';

extension WhenAsyncSnapshotExtension<T> on AsyncSnapshot<T> {
  R when<R>({
    required R Function(T data, bool isLoading) data,
    required R Function(Object, StackTrace?) error,
    required R Function() loading,
  }) {
    if (hasError) {
      return error(this.error!, stackTrace);
    }

    if (hasData) {
      return data(
        this.data!,
        connectionState == ConnectionState.active,
      );
    }

    return loading();
  }

  R maybeWhen<R>({
    R Function(T data, bool isLoading)? data,
    R Function(Object, StackTrace?)? error,
    R Function()? loading,
    required R Function() orElse,
  }) {
    if (hasError && error != null) {
      return error(this.error!, stackTrace);
    }

    if (hasData && data != null) {
      return data(
        this.data!,
        connectionState == ConnectionState.active,
      );
    }

    if (loading != null) {
      return loading();
    }

    return orElse();
  }
}
